﻿using System.Collections.Generic;
using System.Linq;
using Buzzard.Xmpp.Stream.Elements;
using NUnit.Framework;
using System;

namespace Buzzard.Xmpp
{
    [TestFixture]
    public class RosterManagerTest
    {
        #region UpdateRoster method tests

        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void UpdateRoster_IncorrectRosterResultStanza_ThrowException()
        {
            RosterManager manager = new RosterManager();
            IqStanza rosterResult = new IqStanza("");
            manager.UpdateRoster(rosterResult);
        }

        [Test]
        [Ignore]
        public void UpdateRoster_RosterCollectionEmpty_PopulatesRosterCollection()
        {
            RosterManager manager = new RosterManager();
            RosterItem[] items = CreateRosterItems(2);
            IqStanza rosterResult = null;

            manager.UpdateRoster(rosterResult);

            CollectionAssert.AreEquivalent(items, manager.Roster);
        }

        private RosterItem[] CreateRosterItems(int itemCount = 10)
        {
            return Enumerable.Range(0, itemCount).Select(i => new RosterItem("item" + itemCount, "jid" + i + "@domain.com")).ToArray();
        }

        private IqStanza CreateRosterResultStanza(RosterItemElement[] items)
        {
            IqStanza iq = new IqStanza(IqStanzaTypes.Result);
            IqQueryElement query = iq.AddQueryElement(XmppNamespaces.Roster);
            query.AddContentItems(items);
            return iq;
        }

        #endregion

        #region Roster updating tests

        [Test]
        public void AddedItems_AddedItemsToRoster_CollectionContainsAddedItems()
        {
            IEnumerable<RosterItem> addedItems = GenerateRosterItems(4, 5);
            RosterManager manager = CreateRosterManager(1, 2, 3);

            foreach (RosterItem item in addedItems) manager.Roster.Add(item);

            CollectionAssert.AreEquivalent(addedItems, manager.AddedItems);
        }

        [Test]
        public void RemovedItems_RemoveSomeItems_CollectionContainsRemovedItems()
        {
            IEnumerable<RosterItem> removedItems = GenerateRosterItems(2, 4);
            RosterManager manager = CreateRosterManager(1, 2, 3, 4);

            foreach (RosterItem item in removedItems) manager.Roster.Remove(item);
            
            CollectionAssert.AreEquivalent(removedItems, manager.RemovedItems);
        }

        [Test]
        public void ResetRoster_NoArguments_ClearPreviousRoster()
        {
            IEnumerable<RosterItem> rosterItems1 = GenerateRosterItems();
            RosterManager manager = CreateRosterManager();
            foreach (var item in rosterItems1) manager.Roster.Add(item);

            manager.ResetRoster();

            CollectionAssert.IsEmpty(manager.Roster);
        }

        [Test]
        public void ResetRoster_RosterCollectionArgument_ClearPreviousRosterAndSetNewRoster()
        {
            IEnumerable<RosterItem> newRoster = GenerateRosterItems(3, 4, 5);
            RosterManager manager = CreateRosterManager(0, 1, 2);

            manager.ResetRoster(newRoster);

            CollectionAssert.AreEquivalent(newRoster, manager.Roster);
        }

        [Test]
        public void SubmitChanges_RosterChanged_AddedItemsAndRemovedItemsAreEmpty()
        {
            RosterManager manager = CreateRosterManager(1, 2, 3);
            IEnumerable<RosterItem> expectedRoster = GenerateRosterItems(2, 3, 4);
            ChangeRoster(manager);

            manager.SubmitChanges();

            VerifySubmitedChanges(manager, expectedRoster);
        }

        [Test]
        public void RevertChanges_RosterChanged_RestoresPreviousState()
        {
            RosterManager manager = CreateRosterManager(1, 2, 3);
            IEnumerable<RosterItem> expectedRoster = GenerateRosterItems(1, 2, 3);
            ChangeRoster(manager);

            manager.RevertChanges();

            VerifySubmitedChanges(manager, expectedRoster);
        }

        [Test]
        public void PushItem_RosterContainsItems_AddedItemsEmpty()
        {
            RosterManager manager = CreateRosterManager(0, 1);
            RosterItem itemToPush = new RosterItem("item", "some-jid@g.com");

            manager.PushItem(itemToPush);

            Assert.That(manager.AddedItems, Is.Empty);
            Assert.That(manager.RemovedItems, Is.Empty);
            Assert.That(manager.Roster.Contains(itemToPush), Is.True);
        }

        private static void VerifySubmitedChanges(RosterManager manager, IEnumerable<RosterItem> expectedRoster)
        {
            CollectionAssert.IsEmpty(manager.AddedItems);
            CollectionAssert.IsEmpty(manager.RemovedItems);
            CollectionAssert.AreEquivalent(expectedRoster, manager.Roster);
        }

        private void ChangeRoster(RosterManager manager)
        {
            RosterItem itemToAdd = GenerateRosterItems(4).Single();
            RosterItem itemToRemove = GenerateRosterItems(1).Single();
            manager.Roster.Add(itemToAdd);
            manager.Roster.Remove(itemToRemove);
        }

        private RosterManager CreateRosterManager(params int[] itemsIndices)
        {
            IEnumerable<RosterItem> initialItems = GenerateRosterItems(itemsIndices);
            return new RosterManager(initialItems);
        }

        private IEnumerable<RosterItem> GenerateRosterItems(params int[] indices)
        {
            if (indices.Length > 0) return indices.Select(i => new RosterItem("item" + i, "jid" + i + "@domain.com"));
            return Enumerable.Range(1, 3).Select(i => new RosterItem("item" + i, "jid" + i + "@domain.com"));
        }

        #endregion
    }
}
