﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Linq;
using Buzzard.Xmpp.Stream;
using Buzzard.Xmpp.Stream.Elements;
using FakeItEasy;
using NUnit.Framework;

namespace Buzzard.Xmpp
{
    [TestFixture]
    public class XmppClientTests
    {
        #region Roster related tests

        [Test]
        public void SendRosterRequest_RosterCollectionEmpty_SendsRosterGetRequest()
        {
            IStanzaStream stream = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(stream);
            Stanza expectedRequest = CreateRosterRequestStanza();

            client.Connect();
            client.SendRosterRequest();

            A.CallTo(() => stream.Write(expectedRequest)).MustHaveHappened();
        }

        private Stanza CreateRosterRequestStanza()
        {
            IqStanza stanza = new IqStanza();
            stanza.Type = IqStanzaTypes.Get;
            stanza.AddContentItem(new IqQueryElement(XmppNamespaces.Roster));
            return stanza;
        }

        [Test]
        public void Roster_ReceivedRosterResultStanzaWithRosterItems_CollectionContainsReceivedItems()
        {
            IStanzaStream stream = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(stream);
            RosterItemElement[] rosterItems = { new RosterItemElement("item1", "jid1@c.com"), new RosterItemElement("item2", "jid2@c.com") };
            IqStanza rosterResult = CreateRosterResultStanza(rosterItems);
            IEnumerable<RosterItem> expectedItems = rosterItems.Select(ri => (RosterItem)ri);

            client.Connect();
            stream.StanzaReceived += Raise.With(client, new StanzaReceivedEventArgs(rosterResult)).Now;

            CollectionAssert.AreEquivalent(client.Roster, expectedItems);
        }

        private IqStanza CreateRosterResultStanza(RosterItemElement[] items)
        {
            IqStanza rosterResult = new IqStanza(IqStanzaTypes.Result);
            IqQueryElement query = rosterResult.AddQueryElement(XmppNamespaces.Roster);
            query.AddContentItems(items);
            return rosterResult;
        }

        [Test]
        public void Roster_ReceivedRosterPushStanza_NewItemInRoster()
        {
            IStanzaStream stream = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(stream);
            RosterItem itemToPush = new RosterItem("item", "name@domain.com");
            IqStanza rosterPushStanza = CreateRosterPushStanza(itemToPush);

            client.Connect();
            stream.StanzaReceived += Raise.With(client, new StanzaReceivedEventArgs(rosterPushStanza)).Now;

            Assert.That(client.Roster.Contains(itemToPush), Is.True);
        }

        private IqStanza CreateRosterPushStanza(RosterItem itemToPush)
        {
            IqStanza stanza = new IqStanza(IqStanzaTypes.Set);
            stanza.AddQueryElement(XmppNamespaces.Roster)
                  .AddRosterItem(itemToPush.Name, itemToPush.Jid);
            return stanza;
        }

        #endregion

        [Test]
        public void SendSubscriptionRequest_AnyJid_SendsCorrectPresenceStanza()
        {
            IStanzaStream stream = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(stream);
            JID subscriptionTarget = "arnold@s.com";
            PresenceStanza expectedStanza = new PresenceStanza {To = subscriptionTarget, Type = PresenceStanzaType.Subscribe};

            client.Connect();
            client.SendSubscriptionRequest(subscriptionTarget);

            A.CallTo(() => stream.Write(expectedStanza)).MustHaveHappened();
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SendSubscriptionRequest_NullParameter_ThrowsException()
        {
            XmppClient client = CreateClient(A.Fake<IStanzaStream>());
            client.Connect();

            client.SendSubscriptionRequest(null);
        }

        [Test]
        public void SendStanza_AnyStanza_SendsSameStanza()
        {
            IStanzaStream stream = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(stream);
            Stanza stanza = new IqStanza(IqStanzaTypes.Get);

            client.Connect();
            client.SendStanza(stanza);

            A.CallTo(()=>stream.Write(stanza)).MustHaveHappened();
        }

        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SendStanza_NotConnected_ThrowsException()
        {
            XmppClient client = CreateClient(A.Fake<IStanzaStream>());
            client.SendStanza(new IqStanza());
        }

        [Test]
        [ExpectedException(typeof(NullReferenceException))]
        public void SendStanza_NullParam_ThrowsException()
        {
            XmppClient client = CreateClient(A.Fake<IStanzaStream>());
            client.Connect();
            client.SendStanza(null);
        }

        #region Event tests
        [Test]
        public void IqReceived_StreamReturnsIqStanza_RisesEvent()
        {
            IqStanza iq = new IqStanza();
            IqReceivedEventArgs eventArgs = new IqReceivedEventArgs(iq);
            XmppClient client = CreateClient(null);
            VerifyEventRise<IqReceivedEventArgs, IqStanza>("IqReceived", eventArgs);
        }

        [Test]
        public void MessageReceived_StreamReturnsMessageStanza_RisesEvent()
        {
            MessageStanza message = new MessageStanza();
            MessageReceivedEventArgs eventArgs = new MessageReceivedEventArgs(message);
            VerifyEventRise<MessageReceivedEventArgs, MessageStanza>("MessageReceived", eventArgs);
        }

        [Test]
        public void PresenceReceived_StreamReturnsPresenceStanza_RisesEvent()
        {
            PresenceStanza presence = new PresenceStanza();
            PresenceReceivedEventArgs eventArgs = new PresenceReceivedEventArgs(presence);
            VerifyEventRise<PresenceReceivedEventArgs, PresenceStanza>("PresenceReceived", eventArgs);
        }

        private void VerifyEventRise<T,S>(string eventName, T expectedArgs) where T : StanzaEventArgs<S>
                                                                            where S : Stanza
        {
            IStanzaStream streamMock = A.Fake<IStanzaStream>();
            XmppClient client = CreateClient(streamMock);

            bool eventWasRaisen = false;
            EventHandler<T> eventHandler = (s, e) => { eventWasRaisen = true; };
            EventInfo eventInfo = typeof (XmppClient).GetEvent(eventName);
            eventInfo.AddEventHandler(client, eventHandler);

            client.Connect();
            streamMock.StanzaReceived += Raise.With(client, new StanzaReceivedEventArgs(expectedArgs.Stanza)).Now;
            
            Assert.That(eventWasRaisen, Is.True);
        }

        private XmppClient CreateClient(IStanzaStream stanzaStream)
        {
            XmppClient client = A.Fake<XmppClient>();
            A.CallTo(client).Where(oc => oc.Method.Name == "OpenStream")
             .WithReturnType<IStanzaStream>()
             .Returns(stanzaStream);

            return client;
        }

        #endregion
    }
}
