﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace Buzzard.Xmpp.Stream.Elements
{
    public abstract class CompositeElement<T> : XmppStreamElement
    {
        private ICollection<T> items;
        private XmlNamespace contentNamespace;

        #region Constructors

        protected CompositeElement(string name) 
            : base(name)
        {
        }

        protected CompositeElement(string name, string namespaceUri)
            : base(name, namespaceUri)
        {
        }

        protected CompositeElement(string name, string namespaceName, string namespaceUri) 
            : base(name, namespaceName, namespaceUri)
        {
        }

        protected CompositeElement(string name, params T[] children)
            : base(name)
        {
            foreach (T element in children) Content.Add(element);
        }

        #endregion

        public XmlNamespace ContentNamespace
        {
            get { return contentNamespace ?? (contentNamespace = new XmlNamespace("", "")); }
            set
            {
                if (value != null) ValidateContentNamespace(value);
                contentNamespace = value;
            }
        }

        private void ValidateContentNamespace(XmlNamespace ns)
        {
            if (!string.IsNullOrEmpty(ns.Name)) throw new ArgumentException("Content name space must be without name");
        }

        public ICollection<T> Content
        {
            get { return items ?? (items = CreateElementsCollection()); }
        }

        protected virtual ICollection<T> CreateElementsCollection()
        {
            return new List<T>();
        }

        public void AddContentItem(T item)
        {
            Content.Add(item);
        }

        public void AddContentItems(params T[] elements)
        {
            foreach (T element in elements) AddContentItem(element);
        }

        public XmppStreamElement FindElement(string elementName)
        {
            if (elementName == null) throw new ArgumentNullException("elementName");
            if (elementName == "") throw new ArgumentOutOfRangeException("elementName");

            return Content.OfType<XmppStreamElement>().FirstOrDefault(e => e.Name == elementName);
        }

        public TElement FindElement<TElement>(string elementName) where TElement : XmppStreamElement
        {
            return (TElement)FindElement(elementName);
        }

        protected override void WriteStartElement(XmlWriter writer)
        {
            if (ContentNamespace.IsEmpty)
            {
                base.WriteStartElement(writer);
                return;
            }
            if (Namespace.IsEmpty) writer.WriteStartElement(Name, ContentNamespace.Uri);
            if (!Namespace.IsEmpty && !ContentNamespace.IsEmpty)
            {
                base.WriteStartElement(writer);
                writer.WriteAttributeString("xmlns", ContentNamespace.Uri);
            }
        }

        protected override void WritePayload(XmlWriter writer)
        {
            base.WritePayload(writer);
            WriteElements(writer);
        }

        protected abstract void WriteElements(XmlWriter writer);

        public override bool Equals(object obj)
        {
            CompositeElement<T> element = obj as CompositeElement<T>;
            if (element == null) return false;

            return Equals(element);
        }

        public bool Equals(CompositeElement<T> element)
        {
            return base.Equals(element) && AreChildEquals(element);
        }

        private bool AreChildEquals(CompositeElement<T> compositeElement)
        {
            return Content.SequenceEqual(compositeElement.Content);
        }
    }
}
