﻿using System;
using System.Collections.Generic;
using System.Linq;
using Buzzard.Xmpp.Stream;
using Buzzard.Xmpp.Stream.Elements;

namespace Buzzard.Xmpp
{
    public class XmppClient
    {
        private RosterManager rosterManager = new RosterManager();
        private IStanzaStream stanzaStream;

        public bool IsConnected
        {
            get { return stanzaStream != null; }
        }

        public ICollection<RosterItem> Roster
        {
            get{ return rosterManager.Roster; }
        }

        public void Connect()
        {
            stanzaStream = OpenStream();
            stanzaStream.StanzaReceived += StanzaReceivedHandler;
        }

        protected virtual IStanzaStream OpenStream()
        {
            throw new NotImplementedException();
        }

        public void SendStanza(Stanza stanza)
        {
            if (!IsConnected) throw new InvalidOperationException();
            if (stanza == null) throw new NullReferenceException(); 
            stanzaStream.Write(stanza);
        }

        public void SendRosterRequest()
        {
            Stanza rosterRequest = CreateRosterRequestStanza();
            SendStanza(rosterRequest);
        }

        private Stanza CreateRosterRequestStanza()
        {
            IqStanza request = new IqStanza();
            request.AddContentItem(new IqQueryElement(XmppNamespaces.Roster));
            request.Type = IqStanzaTypes.Get;
            return request;
        }

        public void SendSubscriptionRequest(JID subscriptionTarget)
        {
            if (subscriptionTarget == null) throw new ArgumentNullException("subscriptionTarget");
            Stanza subscriptionRequest = CreateSubscriptionRequest(subscriptionTarget);
            stanzaStream.Write(subscriptionRequest);
        }

        private Stanza CreateSubscriptionRequest(JID subscriptionTarget)
        {
            PresenceStanza subscriptionRequest = new PresenceStanza();
            subscriptionRequest.Type = PresenceStanzaType.Subscribe;
            subscriptionRequest.To = subscriptionTarget;
            return subscriptionRequest;
        }

        private void StanzaReceivedHandler(object sender, StanzaReceivedEventArgs args)
        {
            Stanza stanza = args.Stanza;
            if (stanza is IqStanza) OnIqReceived((IqStanza) stanza);
            if (stanza is MessageStanza) OnMessageReceived((MessageStanza)stanza);
            if (stanza is PresenceStanza) OnPresenceReceived((PresenceStanza)stanza);
        }

        private void OnIqReceived(IqStanza iq)
        {
            if (IqReceived != null) IqReceived(this, new IqReceivedEventArgs(iq));
            if (iq.IsRosterResult())
            {
                var rosterItems = iq.GetRosterItemsElements().Select(ri => (RosterItem)ri);
                rosterManager.ResetRoster(rosterItems);
            }
            if (iq.Type == IqStanzaTypes.Set && iq.HasRosterQuery())
            {
                RosterItem rosterItem = iq.GetRosterItemsElements().Single();
                rosterManager.PushItem(rosterItem);
            }
        }

        private void OnMessageReceived(MessageStanza message)
        {
            if (MessageReceived != null) MessageReceived(this, new MessageReceivedEventArgs(message));
        }

        private void OnPresenceReceived(PresenceStanza presence)
        {
            if (PresenceReceived != null) PresenceReceived(this, new PresenceReceivedEventArgs(presence));
        }

        public event EventHandler<IqReceivedEventArgs> IqReceived;

        public event EventHandler<MessageReceivedEventArgs> MessageReceived;

        public event EventHandler<PresenceReceivedEventArgs> PresenceReceived;
    }
}
